﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace aFrameEdit
{
    public partial class GroupSaveAsForm : Form
    {
        private const int DspMemoryGrp = 8;
        private int seledGrp = 0;

        public GroupSaveAsForm(int group, ProjectGroupList grpList, ProjectToneNameList toneNameList)
        {
            InitializeComponent();
            seledGrp = group;
            setDataGridView(group, grpList, toneNameList);
        }

        private void setDataGridView(int group, ProjectGroupList grpList, ProjectToneNameList toneNameList)
        {
            int max = grpList.gMax;

            // Prepare Columns
            dataGridView_grp_saveAs.Columns.Add("GroupNum", "Group Num");
            dataGridView_grp_saveAs.Columns.Add("SeledInstTone", "Selected Inst Tone");
            dataGridView_grp_saveAs.Columns.Add("SeledEffectTone", "Selected Effect Tone");
            dataGridView_grp_saveAs.Columns[0].Width = 60;
            dataGridView_grp_saveAs.Columns[1].Width = 120;
            dataGridView_grp_saveAs.Columns[2].Width = 120;

            this.dataGridView_grp_saveAs.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.None;

            // Prepare Rows
            for (int j = 0; j < max-1; j++)
            {
                dataGridView_grp_saveAs.Rows.Add();
            }

            // Set Group and Number 
            for (int j = 0; j < max; j++)
            {
                dataGridView_grp_saveAs[0, j].Value = getGroupString(group) + (j+1).ToString("00");
                dataGridView_grp_saveAs[1, j].Value = toneNameList.instNames[grpList.instNum[j]];
                dataGridView_grp_saveAs[2, j].Value = toneNameList.effectNames[grpList.effectNum[j]];
            }

            this.dataGridView_grp_saveAs.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells;
            this.dataGridView_grp_saveAs.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            this.dataGridView_grp_saveAs.MultiSelect = false;
        }

        private string getGroupString(int group)
        {
            string grpString;
            int grpNum = group;

            if (group >= 4)
            {
                grpNum -= 4;
                grpString = (grpNum + 0xA).ToString("X") + "'";
            }
            else
            {
                grpString = (grpNum + 0xA).ToString("X") + "-";
            }
            return grpString;
        }

        private void button_group_saveAs_prev_Click(object sender, EventArgs e)
        {
            MainForm parent = (MainForm)this.Owner;
            int grp = seledGrp;
            int prevGrp;
            int num = dataGridView_grp_saveAs.CurrentRow.Index;

            if (grp == 0)
            {
                prevGrp = DspMemoryGrp - 1;
            }
            else
            {
                prevGrp = grp - 1;
            }
            
            parent.getProjectGroupList(prevGrp);
            dataGridView_grp_saveAs.Rows.Clear();
            dataGridView_grp_saveAs.Columns.Clear();
            setDataGridView(prevGrp, parent.prjGroupList, parent.prjToneNameList);           
            seledGrp = prevGrp;

            if (num < parent.prjGroupList.gMax - 1)
            {
                button_group_saveAs_add.Enabled = false;
                button_group_saveAs_remove.Enabled = false;
                button_group_saveAs_add.Visible = false;
                button_group_saveAs_remove.Visible = false;
            }
        }

        private void button_group_saveAs_next_Click(object sender, EventArgs e)
        {
            MainForm parent = (MainForm)this.Owner;
            int grp = seledGrp;
            int nextGrp;
            int num = dataGridView_grp_saveAs.CurrentRow.Index;

            if (grp == DspMemoryGrp - 1)
            {
                nextGrp = 0;
            }
            else
            {
                nextGrp = grp + 1;
            }
            parent.getProjectGroupList(nextGrp);
            dataGridView_grp_saveAs.Rows.Clear();
            dataGridView_grp_saveAs.Columns.Clear();
            setDataGridView(nextGrp, parent.prjGroupList, parent.prjToneNameList);
            seledGrp = nextGrp;

            if (num < parent.prjGroupList.gMax - 1)
            {
                button_group_saveAs_add.Enabled = false;
                button_group_saveAs_remove.Enabled = false;
                button_group_saveAs_add.Visible = false;
                button_group_saveAs_remove.Visible = false;
            }
        }

        private void button_group_saveAs_save_Click(object sender, EventArgs e)
        {
            MainForm parent = (MainForm)this.Owner;
            int num = dataGridView_grp_saveAs.CurrentRow.Index;
            int max = parent.prjGroupList.gMax;

            parent.writeGroup(seledGrp, num, max);
            parent.isTonePairChanged = false;
            parent.setToneNameColor();
            parent.changeGroupInternal(seledGrp, num);
            parent.disableGroupOriginalAndSaveButton();
            parent.setGroupOriginalButtonText("Original");
            this.Dispose();
        }

        private void button_group_saveAs_add_Click(object sender, EventArgs e)
        {
            MainForm parent = (MainForm)this.Owner;
            int num = dataGridView_grp_saveAs.CurrentRow.Index;
            int max = parent.prjGroupList.gMax;

            parent.writeGroup(seledGrp, num + 1, max + 1);
            parent.isTonePairChanged = false;
            parent.setToneNameColor();
            parent.changeGroupInternal(seledGrp, num + 1);
            parent.disableGroupOriginalAndSaveButton();
            parent.setGroupOriginalButtonText("Original");
            this.Dispose();
        }

        private void button_group_saveAs_remove_Click(object sender, EventArgs e)
        {
            MainForm parent = (MainForm)this.Owner;
            int num = dataGridView_grp_saveAs.CurrentRow.Index;
            int max = parent.prjGroupList.gMax;

            parent.changeGroupInternal(seledGrp, num - 1);
            parent.writeGroup(seledGrp, num - 1, max - 1);
            parent.isTonePairChanged = false;
            parent.setToneNameColor();
            parent.changeGroupInternal(seledGrp, num - 1);
            parent.disableGroupOriginalAndSaveButton();
            parent.setGroupOriginalButtonText("Original");
            this.Dispose();
        }

        private void dataGridView_grp_saveAs_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            MainForm parent = (MainForm)this.Owner;

            if (e.RowIndex == parent.prjGroupList.gMax - 1)
            {
                button_group_saveAs_add.Enabled = true;
                button_group_saveAs_remove.Enabled = true;
                button_group_saveAs_add.Visible = true;
                button_group_saveAs_remove.Visible = true;
            }
            else
            {
                button_group_saveAs_add.Enabled = false;
                button_group_saveAs_remove.Enabled = false;
                button_group_saveAs_add.Visible = false;
                button_group_saveAs_remove.Visible = false;
            }
        }
    }
}
