﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.IO.Ports;
using System.Windows.Forms;
using System.Management;
using System.Threading;

namespace aFrameEdit
{
    public partial class MainForm
    {
        private const string aFrameComDeviceName = "Silicon Labs CP210x USB to UART Bridge ";

        private string aFramePortName; // port name.
        private string serialRcvData;
        private bool isSerialTimerout;
        private volatile bool isSerialRcved;
        private bool isDevFound = false;
        private bool is1st = false;

        // COM PORT
        private void InitComPort()
        {
            aFramePortName = "";
            Dictionary<string, string> portList = new Dictionary<string, string>();
            ManagementClass mcW32SerPort = new ManagementClass("Win32_SerialPort");
       
            foreach (ManagementObject port in mcW32SerPort.GetInstances())
            {
                string caption = (string)port.GetPropertyValue("Caption");  // ex. Silicon Labs CP210x USB to UART Bridge (COM3)
                string[] captions = caption.Split('(');
                string devName = captions[0];
                string devId = (string)port.GetPropertyValue("DeviceID");   // ex. COM3
                portList.Add(devName, devId);
            }
            if (portList.Count > 0)
            {
                try
                {
                    aFramePortName = portList[aFrameComDeviceName];
                }
                catch (KeyNotFoundException)
                {
#if DEBUG
                    MessageBox.Show("KEY NOT FOUND");
#endif
                }
            }

            if (aFramePortName != "")
            {
                toolStripButton_Con.Enabled = true;
                isDevFound = true;
                timer_con.Enabled = false;
            }
        }

        private void connectComPort()
        {
            isSerialRcved = false;
            isSerialTimerout = false;
            serialRcvData = null;
            try
            {
                if (is1st == false)
                {
                    serialPort_aFrame.PortName = aFramePortName;
                    serialPort_aFrame.BaudRate = 115200;        // Baudrate: 115200 bps
                    serialPort_aFrame.DataBits = 8;             // Data: 8 bit
                    serialPort_aFrame.Parity = Parity.None;     // Parity: None
                    serialPort_aFrame.StopBits = StopBits.One;  // Stop: 1 bit
                    serialPort_aFrame.NewLine = Environment.NewLine;
                    serialPort_aFrame.DataReceived += new SerialDataReceivedEventHandler(serialPortDataReceived);
                }
                if (serialPort_aFrame.IsOpen == false)
                {
                    serialPort_aFrame.Open();
                    is1st = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void serialPortDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                serialRcvData = serialPort_aFrame.ReadExisting();
                if (serialRcvData != "")
                {
                    isSerialRcved = true;
                }
                else
                {
                    MessageBox.Show("received empty string !!!");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private string getStringfromRcvdData()
        {
            if (serialRcvData != "")
            {
                return serialRcvData;
            }
            else
            {
                return null;
            }
        }

        private string[] getStringArrayfromRcvdData()
        {
            if (serialRcvData != "")
            {
                return serialRcvData.Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
            }
            else
            {
                return null;
            }
        }

        private void disconnectComPort()
        {
            try
            {
                if (serialPort_aFrame.IsOpen == true) // Port checking, if port is opened, Close.
                {
                    serialPort_aFrame.Close();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void writeData(string data)
        {
            try
            {
                serialPort_aFrame.Write(data);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }   
}
